from __future__ import with_statement
import Live
from _Framework.ControlSurface import ControlSurface
from _Framework.InputControlElement import *
from _Framework.ButtonElement import ButtonElement
from _Framework.ButtonMatrixElement import ButtonMatrixElement

from ConfigurableButtonElement import ConfigurableButtonElement
from MainSelectorComponent import MainSelectorComponent
SIDE_NOTES = (8, 24, 40, 56, 72, 88, 104, 120)
DRUM_NOTES = (41, 42, 43, 44, 45, 46, 47, 57, 58, 59, 60, 61, 62, 63, 73, 74, 75, 76, 77, 78, 79, 89, 90, 91, 92, 93, 94, 95, 105, 106, 107)

log_enabled = False

class Loungepet(ControlSurface):
    """ Script for Loungepet Controller """

    def __init__(self, c_instance):
        ControlSurface.__init__(self, c_instance)
        if(log_enabled):
			Live.Base.log("Loungepet - Init ")
        with self.component_guard():
            self._set_suppress_features(True)
            is_momentary = True
            self._suggested_input_port = 'Launchpad'
            self._suggested_output_port = 'Launchpad'
            self._control_is_with_automap = False
            self._user_byte_write_button = ButtonElement(is_momentary, MIDI_CC_TYPE, 0, 16)
            self._user_byte_write_button.name = 'User_Byte_Button'
            self._user_byte_write_button.send_value(1)
            self._user_byte_write_button.add_value_listener(self._user_byte_value)
            self._wrote_user_byte = False
            
            matrix = ButtonMatrixElement()
            matrix.name = 'Button_Matrix'
            for row in range(8):
                button_row = []
                for column in range(8):
                    button = ConfigurableButtonElement(is_momentary, MIDI_NOTE_TYPE, 0, row * 16 + column)
                    button.name = str(column) + '_Clip_' + str(row) + '_Button'
                    button_row.append(button)

                matrix.add_row(tuple(button_row))

            self._config_button = ButtonElement(is_momentary, MIDI_CC_TYPE, 0, 0, optimized_send_midi=False)
            self._config_button.add_value_listener(self._config_value)
            
            top_buttons = [ ConfigurableButtonElement(is_momentary, MIDI_CC_TYPE, 0, 104 + index) for index in range(8) ]
            
            side_buttons = [ ConfigurableButtonElement(is_momentary, MIDI_NOTE_TYPE, 0, SIDE_NOTES[index]) for index in range(8) ]
            
            top_buttons[0].name = 'Bank_Select_Up_Button'
            top_buttons[1].name = 'Bank_Select_Down_Button'
            top_buttons[2].name = 'Bank_Select_Left_Button'
            top_buttons[3].name = 'Bank_Select_Right_Button'
            top_buttons[4].name = 'Session_Button'
            top_buttons[5].name = 'User1_Button'
            top_buttons[6].name = 'User2_Button'
            top_buttons[7].name = 'Mixer_Button'
            
            side_buttons[0].name = 'Vol_Button'
            side_buttons[1].name = 'Pan_Button'
            side_buttons[2].name = 'SndA_Button'
            side_buttons[3].name = 'SndB_Button'
            side_buttons[4].name = 'Stop_Button'
            side_buttons[5].name = 'Trk_On_Button'
            side_buttons[6].name = 'Solo_Button'
            side_buttons[7].name = 'Arm_Button'

            self._selector = MainSelectorComponent(matrix, tuple(top_buttons), tuple(side_buttons), self._config_button)
            self._selector.name = 'Main_Modes'
            for control in self.controls:
                if isinstance(control, ConfigurableButtonElement):
                    control.add_value_listener(self._button_value)

            self._set_suppress_features(False)        
            self.set_highlighting_session_component(self._selector.session_component())
            self._suppress_session_highlight = False
        if(log_enabled):
			Live.Base.log("Loungepet - Init End")

    def _set_suppress_features(self, state):
        if(log_enabled):
			Live.Base.log("Loungepet - _set_suppress_features " + str(state))
        self._suppress_session_highlight = state # enable/disable the "Red box"
        self._suppress_send_midi = state # sounds like a good idea
        self._set_suppress_rebuild_requests(state)            
            
    def disconnect(self):
        if(log_enabled):
			Live.Base.log("Loungepet - disconnect ")
        self._suppress_send_midi = True
        for control in self.controls:
            if isinstance(control, ConfigurableButtonElement):
                control.remove_value_listener(self._button_value)

        self._selector = None
        self._user_byte_write_button.remove_value_listener(self._user_byte_value)
        self._config_button.remove_value_listener(self._config_value)
        ControlSurface.disconnect(self)
        self._suppress_send_midi = False
        self._config_button.send_value(32) # Turn flashing LEDs on
        self._config_button.send_value(0) # Reset Launchpad
        self._config_button = None
        self._user_byte_write_button.send_value(0)
        self._user_byte_write_button = None

    def refresh_state(self):
        if(log_enabled):
			Live.Base.log("Loungepet - refresh_state Start")        
        ControlSurface.refresh_state(self)
        self.schedule_message(5, self._update_hardware)
        if(log_enabled):
			Live.Base.log("Loungepet - refresh_state End")

    def handle_sysex(self, midi_bytes):
        if(log_enabled):
			Live.Base.log("Loungepet - handle_sysex - midi_bytes=" + str(midi_bytes))        
        self._suppress_send_midi = False
        self.set_enabled(True)

    def build_midi_map(self, midi_map_handle):
        if(log_enabled):
			Live.Base.log("Loungepet - build_midi_map - midi_map_handle=" + str(midi_map_handle))         
        ControlSurface.build_midi_map(self, midi_map_handle)
        if self._selector.mode_index == 1:
            new_channel = self._selector.channel_for_current_mode()
            for note in DRUM_NOTES:
                self._translate_message(MIDI_NOTE_TYPE, note, 0, note, new_channel)
        if(log_enabled):
			Live.Base.log("Loungepet - build_midi_map End")

    def _send_midi(self, midi_bytes, optimized = None):
        if(log_enabled):
			Live.Base.log("Loungepet - _send_midi - midi_map_handle=" + str(midi_bytes))         
        sent_successfully = False
        if not self._suppress_send_midi:
            sent_successfully = ControlSurface._send_midi(self, midi_bytes, optimized=optimized)
        if(log_enabled):
			Live.Base.log("Loungepet - _send_midi End returning=" + str(sent_successfully))
        return sent_successfully

    def _update_hardware(self):
        if(log_enabled):
			Live.Base.log("Loungepet - _update_hardware Start")
        self._suppress_send_midi = False
        self._wrote_user_byte = True
        self._user_byte_write_button.send_value(1)
        self.set_enabled(True)
        if(log_enabled):
			Live.Base.log("Loungepet - _update_hardware End")

    def _user_byte_value(self, value):
        if(log_enabled):
			Live.Base.log("Loungepet - _user_byte_value Start")
        if not value in range(128):
            raise AssertionError
            enabled = self._wrote_user_byte or value == 1
            self._control_is_with_automap = not enabled
            self._suppress_send_midi = self._control_is_with_automap
            if not self._control_is_with_automap:
                for control in self.controls:
                    if isinstance(control, ConfigurableButtonElement):
                        control.set_force_next_value()

            self._selector.set_mode(0)
            self.set_enabled(enabled)
            self._suppress_send_midi = False
        else:
            self._wrote_user_byte = False
        if(log_enabled):
			Live.Base.log("Loungepet - _user_byte_value End")

    def _button_value(self, value):
        if(log_enabled):
			Live.Base.log("Loungepet - _button_value - value=" + str(value))   
        assert value in range(128)

    def _config_value(self, value):
        if(log_enabled):
			Live.Base.log("Loungepet - _config_value - value=" + str(value)) 
        assert value in range(128)

    def _set_session_highlight(self, track_offset, scene_offset, width, height, include_return_tracks):
        if(log_enabled):
			Live.Base.log("Loungepet - _set_session_highlight - track_offset=" + str(track_offset) + " scene_offset=" + str(scene_offset) + " width=" + str(width) + " height=" + str(height) + " include_return_tracks=" + str(include_return_tracks))        
        if not self._suppress_session_highlight:
            ControlSurface._set_session_highlight(self, track_offset, scene_offset, width, height, include_return_tracks)
        if(log_enabled):
			Live.Base.log("Loungepet - _set_session_highlight End")